﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Bonea.PropertyRegister.Common;
using Microsoft.Web.Services3;

namespace Bonea.PropertyRegister.WinFormsCommon
{
    public partial class EditFormBase : Form
    {
        public EditFormBase()
        {
            InitializeComponent();
        }

        protected virtual void BindToObject(object entity)
        {
            return;
        }

        protected virtual void SetControlStatus()
        {
            return;
        }

        protected virtual void OnBtnSave(object sender, EventArgs e)
        {
            return;
        }

        protected object mEntity;

        public virtual DialogResult Execute(object entity)
        {
            this.Cursor = Cursors.WaitCursor;
            try
            {
                mEntity = entity;
                BindToObject(entity);
                SetControlStatus();
            }
            finally
            {
                this.Cursor = Cursors.Default;
            }

            DialogResult dlgResult = ShowDialog();
            return dlgResult;
        }

        private void btnSave_Click(object sender, EventArgs e)
        {
            try
            {
                this.Cursor = Cursors.WaitCursor;
                try
                {
                    OnBtnSave(sender, e);

                    DialogResult = DialogResult.OK;
                }
                finally
                {
                    this.Cursor = Cursors.Default;
                }
            }
            catch (ResponseProcessingException wseException)
            {
                if ((wseException.InnerException != null) && 
                    (wseException.InnerException is RuleException))
                {
                    RuleException ruleException = 
                        (RuleException)wseException.InnerException;
                    BindErrorsToControls(this.Controls, customEditErrorProvider,
                        ruleException.ErrorsList, btnSave);
                }

                throw wseException;
            }
            catch (RuleException ruleException)
            {
                BindErrorsToControls(this.Controls, customEditErrorProvider,
                    ruleException.ErrorsList, btnSave);

                throw ruleException;
            }		
        }

        protected void BindTextBox(TextBox textControl, object dataSource,
            string dataMember)
        {
            textControl.DataBindings.Clear();
            textControl.DataBindings.Add("Text", dataSource, dataMember);
        }

        protected void BindComboBox(ComboBox combo, string propertyName,
            object bindingSource, string bindingMember, string displayMember,
            string valueMember, object dataSource)
        {
            combo.DataSource = dataSource;
            combo.DisplayMember = displayMember;
            combo.ValueMember = valueMember;
            combo.DataBindings.Clear();
            combo.DataBindings.Add(propertyName, bindingSource, bindingMember, false);
        }

        protected void BindDateTimePicker(DateTimePicker dateTimePickerControl,
            object dataSource, string dataMember)
        {
            dateTimePickerControl.DataBindings.Clear();
            dateTimePickerControl.DataBindings.Add("Value", dataSource, dataMember);
        }

        private static void BindErrorsToControls(
            Control.ControlCollection controlCollection,
            ErrorProvider errorProvider,
            List<RuleError> errorsList,
            Control defaultErrorControl)
        {
            List<Control> relatedControlsToError = new List<Control>();

            ResetControlsErrorStatus(controlCollection, errorProvider);

            for (int errorIndex = 0; errorIndex < errorsList.Count;
                errorIndex++)
            {

                relatedControlsToError.Clear();
                GetRelatedControlsToError(controlCollection,
                    ref relatedControlsToError,
                    errorsList[errorIndex].propertyName);
                if (relatedControlsToError.Count == 0)
                {
                    if (defaultErrorControl != null)
                    {
                        errorProvider.SetError(defaultErrorControl,
                            errorsList[errorIndex].ErrorMessage);
                    }
                }
                else
                {
                    foreach (Control control in relatedControlsToError)
                    {
                        errorProvider.SetError(control,
                            errorsList[errorIndex].ErrorMessage);
                    }
                }

            }
        }

        private static void ResetControlsErrorStatus(
            Control.ControlCollection controlCollection,
            ErrorProvider errorProvider)
        {
            foreach (Control control in controlCollection)
            {
                errorProvider.SetError(control, "");
                if (control.Controls.Count > 0)
                {
                    ResetControlsErrorStatus(control.Controls,
                        errorProvider);
                }
            }
        }

        private static void GetRelatedControlsToError(
            Control.ControlCollection controlCollection,
            ref List<Control> relatedControlsList, string errorPropertyName)
        {
            if (relatedControlsList == null)
            {
                relatedControlsList = new List<Control>();
            }

            int bindingsCount;
            object[] bindingsArray;
            Binding currentBinding;

            foreach (Control control in controlCollection)
            {
                if (control.DataBindings.Count > 0)
                {
                    bindingsCount = control.DataBindings.Count;
                    bindingsArray = new object[bindingsCount];
                    control.DataBindings.CopyTo(bindingsArray, 0);

                    for (int bindingsIndex = 0;
                        bindingsIndex < bindingsArray.Length; bindingsIndex++)
                    {
                        currentBinding =
                            (Binding)bindingsArray[bindingsIndex];
                        if (errorPropertyName ==
                            currentBinding.BindingMemberInfo.BindingField)
                        {
                            relatedControlsList.Add(control);
                            break;
                        }
                    }
                }

                if (control.Controls.Count > 0)
                {
                    GetRelatedControlsToError(control.Controls,
                        ref relatedControlsList, errorPropertyName);
                }
            }
        }
    }
}