﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Web;
using NHibernate;

namespace Bonea.PropertyRegister.DataAccess
{
    public class PerRequestSessionManager : SessionManager, IHttpModule
    {
        internal PerRequestSessionManager()
        {
        }

        private const string NHIBERNATE_SESSION_KEY = "NHIBERNATE_SESSION";

        public override ISession Session
        {
            get
            {
                ISession resultSession;

                if (HttpContext.Current.Items.Contains(NHIBERNATE_SESSION_KEY))
                {
                    resultSession =
                        (ISession)HttpContext.Current.Items[NHIBERNATE_SESSION_KEY];
                }
                else
                {
                    resultSession = NewSession();
                    HttpContext.Current.Items[NHIBERNATE_SESSION_KEY] = resultSession;
                }

                return resultSession;
            }
        }

        public override bool IsActiveSession
        {
            get { return HttpContext.Current.Items.Contains(NHIBERNATE_SESSION_KEY); }
        }

        public override void CloseSession()
        {
            if (IsActiveSession == true)
            {
                if (IsActiveTransaction == true)
                {
                    RollbackTransaction();
                }

                Session.Close();
                HttpContext.Current.Items.Remove(NHIBERNATE_SESSION_KEY);
            }
        }

        #region IHttpModule

        public void Init(HttpApplication context)
        {
            context.EndRequest += new EventHandler(OnEndRequest);
        }

        public void Dispose()
        {
            //Intentionaly not implemented
        }

        public void OnEndRequest(Object sender, EventArgs e)
        {
            CloseSession();
        }

        #endregion
    }
}
