﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Bonea.PropertyRegister.MunicipalityProperty.WindowsClient.PropertyCertificateReference;
using web = Bonea.PropertyRegister.MunicipalityProperty.WindowsClient.PropertyCertificateReference;
using Bonea.PropertyRegister.WinFormsCommon;
using Bonea.PropertyRegister.Common;
using System.Security.Permissions;

namespace Bonea.PropertyRegister.MunicipalityProperty.WindowsClient
{
    public partial class PropertyCertificateEditForm : EditFormBase
    {
        public PropertyCertificateEditForm()
        {
            InitializeComponent();

            service = ServiceFactory.GetPropertyCertificateService();
        }

        PropertyCertificateServiceWse service;

        protected PropertyCertificate Entity
        {
            get { return (PropertyCertificate)mEntity; }
            set { mEntity = value; }
        }

        protected override void BindToObject(object entity)
        {
            base.BindToObject(entity);

            BindTextBox(txtNumber, entity, "Number");
            BindTextBox(txtStatus, entity, "StatusName");
            BindDateTimePicker(dtpDateCreated, entity, "DateCreated");
            BindTextBox(txtLawReason, entity, "LawReason");
            BindTextBox(txtRegisterNumber, entity, "RegisterNumber");
            BindTextBox(txtCardIndexNumber, entity, "CardIndexNumber");
            BindTextBox(txtFileNumber, entity, "FileNumber");
            BindTextBox(txtDescription, entity, "Description");
            BindTextBox(txtLocation, entity, "Location");
            BindTextBox(txtBorders, entity, "Borders");
            BindTextBox(txtTaxAssesment, entity, "TaxAssesment");
            BindTextBox(txtMarketAssesment, entity, "MarketAssesment");
            BindTextBox(txtExOwners, entity, "ExOwners");
            BindTextBox(txtOwners, entity, "Owners");
            BindTextBox(txtExCertificates, entity, "ExCertificates");
            BindTextBox(txtOrders, entity, "Orders");
            BindTextBox(txtNotes, entity, "Notes");
            BindTextBox(txtRejectRemarks, entity, "RejectRemarks");
            BindTextBox(txtRegistrationNumber, entity, "RegistrationNumber");
            BindDateTimePicker(dtpRegistrationDate, entity, "RegistrationDate");
            BindTextBox(txtCreatedByUser, entity, "CreatedByUserFullname");
            BindTextBox(txtApprovedByUser, entity, "ApprovedByUserFullname");
            BindTextBox(txtSignedByUser, entity, "SignedByUserFullname");
            BindTextBox(txtLastUpdatedByUser, entity, "LastUpdatedByUserFullname");
            BindComboBox(cbType, "SelectedItem", entity, "DocumentType", "Name", "ID",
                service.GetTypeList());
            BindComboBox(cbIssuer, "SelectedItem", entity, "Issuer", "Name", "ID",
                service.GetIssuerList());
        }

        protected override void SetControlStatus()
        {
            base.SetControlStatus();

            bool canSave = AppUtils.HasPermission(
                UserCredentials.PropertyCertificateSave);
            bool documentCanBeChanged =
                (Entity.Status.StatusCode == web.PropertyCertificateStatusCodes.Created) ||
                (Entity.Status.StatusCode == web.PropertyCertificateStatusCodes.ApprovalRejected) ||
                (Entity.Status.StatusCode == web.PropertyCertificateStatusCodes.SignRejected);

            //Enable/disable edit controls
            txtNumber.ReadOnly = ! (canSave && documentCanBeChanged);
            dtpDateCreated.Enabled = canSave && documentCanBeChanged;
            txtLawReason.ReadOnly = ! (canSave && documentCanBeChanged);
            txtRegisterNumber.ReadOnly = ! (canSave && documentCanBeChanged);
            txtCardIndexNumber.ReadOnly = ! ( canSave && documentCanBeChanged);
            txtFileNumber.ReadOnly = ! ( canSave && documentCanBeChanged);
            txtDescription.ReadOnly = ! ( canSave && documentCanBeChanged);
            txtLocation.ReadOnly = ! ( canSave && documentCanBeChanged);
            txtBorders.ReadOnly = ! ( canSave && documentCanBeChanged);
            txtTaxAssesment.ReadOnly = ! ( canSave && documentCanBeChanged);
            txtMarketAssesment.ReadOnly = ! ( canSave && documentCanBeChanged);
            txtExOwners.ReadOnly = ! ( canSave && documentCanBeChanged);
            txtOwners.ReadOnly = ! ( canSave && documentCanBeChanged);
            txtExCertificates.ReadOnly = ! ( canSave && documentCanBeChanged);
            txtOrders.ReadOnly = ! ( canSave && documentCanBeChanged);
            txtNotes.ReadOnly = !(canSave && (Entity.Status.StatusCode != web.PropertyCertificateStatusCodes.Registered));
            txtRejectRemarks.ReadOnly = ! ( canSave && documentCanBeChanged);
            txtRegistrationNumber.ReadOnly = ! ( canSave && documentCanBeChanged);
            dtpRegistrationDate.Enabled = canSave && documentCanBeChanged;
            cbType.Enabled = canSave && documentCanBeChanged;
            cbIssuer.Enabled = canSave && documentCanBeChanged;

            //Enable/disable buttons
            btnSendForApproval.Enabled = AppUtils.HasPermission(UserCredentials.PropertyCertificateSendForApproval) &&
                ((Entity.Status.StatusCode == web.PropertyCertificateStatusCodes.Created) ||
                (Entity.Status.StatusCode == web.PropertyCertificateStatusCodes.ApprovalRejected) ||
                (Entity.Status.StatusCode == web.PropertyCertificateStatusCodes.SignRejected));
            btnApprove.Enabled = AppUtils.HasPermission(UserCredentials.PropertyCertificateApprove) &&
                (Entity.Status.StatusCode == web.PropertyCertificateStatusCodes.SendedForApproval);
            btnSign.Enabled = AppUtils.HasPermission(UserCredentials.PropertyCertificateSign) &&
                (Entity.Status.StatusCode == web.PropertyCertificateStatusCodes.Approved);
            btnReject.Enabled = (AppUtils.HasPermission(UserCredentials.PropertyCertificateApprove) || 
                AppUtils.HasPermission(UserCredentials.PropertyCertificateSign)) &&
                ((Entity.Status.StatusCode == web.PropertyCertificateStatusCodes.SendedForApproval) ||
                (Entity.Status.StatusCode == web.PropertyCertificateStatusCodes.Approved));
            btnSave.Enabled = canSave ;
            btnDelete.Enabled = AppUtils.HasPermission(UserCredentials.PropertyCertificateDelete) &&
                (Entity.ID != AppConst.UnknownID) &&
                ((Entity.Status.StatusCode == web.PropertyCertificateStatusCodes.ApprovalRejected) ||
                (Entity.Status.StatusCode == web.PropertyCertificateStatusCodes.Created));
            
            bool propertyCertificateIsSavedToDB = this.Entity.ID != AppConst.UnknownID;
            btnScan.Enabled = AppUtils.HasPermission( UserCredentials.PropertyCertificateView ) && 
                propertyCertificateIsSavedToDB;
        }

        [PrincipalPermission(SecurityAction.Demand,
           Role = UserCredentials.PropertyCertificateSave, Authenticated = true)]
        protected override void OnBtnSave(object sender, EventArgs e)
        {
            base.OnBtnSave(sender, e);

            service.Save((PropertyCertificate)mEntity);
        }

        [PrincipalPermission(SecurityAction.Demand,
           Role = UserCredentials.PropertyCertificateSendForApproval,
            Authenticated = true)]        
        private void btnSendForApproval_Click(object sender, EventArgs e)
        {
            if (ConfirmationDialog.Execute(
                "Текущият акт ще бъде изпратен за проверка. Желаете ли да продължите?",
                "Изпращане за проверка на акт") == false)
            {
                return;
            }

            this.Cursor = Cursors.WaitCursor;
            try
            {
                Entity = service.ChangeStatus(Entity,
                    web.PropertyCertificateStatusCodes.SendedForApproval);

                BindToObject(Entity);
                Entity.RefreshBindedControls();
                SetControlStatus();
            }
            finally
            {
                this.Cursor = Cursors.Default;
            }
        }

        [PrincipalPermission(SecurityAction.Demand,
            Role = UserCredentials.PropertyCertificateApprove, Authenticated = true)]
        private void btnApprove_Click(object sender, EventArgs e)
        {
            if (ConfirmationDialog.Execute(
                "Текущият акт ще бъде установен в състояние 'Проверен'. Желаете ли да продължите?",
                "Проверка на акт") == false)
            {
                return;
            }

            this.Cursor = Cursors.WaitCursor;
            try
            {
                Entity = service.ChangeStatus(Entity,
                    web.PropertyCertificateStatusCodes.Approved);

                BindToObject(Entity);
                Entity.RefreshBindedControls();
                SetControlStatus();
            }
            finally
            {
                this.Cursor = Cursors.Default;
            }
        }

        [PrincipalPermission(SecurityAction.Demand,
           Role = UserCredentials.PropertyCertificateSign, Authenticated = true)]        
        private void btnSign_Click(object sender, EventArgs e)
        {
            if (ConfirmationDialog.Execute(
                "Текущият акт ще бъде установен в състояние 'Подписан'. Желаете ли да продължите?",
                "Подписване на акт") == false)
            {
                return;
            }

            this.Cursor = Cursors.WaitCursor;
            try
            {
                Entity = service.ChangeStatus(Entity,
                    web.PropertyCertificateStatusCodes.Signed);
                
                BindToObject(Entity);
                Entity.RefreshBindedControls();
                SetControlStatus();
            }
            finally
            {
                this.Cursor = Cursors.Default;
            }
        }

        //[PrincipalPermission(SecurityAction.Demand,
        //   Role = UserCredentials.PropertyCertificateCreate, Authenticated = true)]        
        //TODO: Invoke security demand via code depending of the document status
        private void btnReject_Click(object sender, EventArgs e)
        {
            if (ConfirmationDialog.Execute(
                "Текущият акт ще бъде установен в състояние 'Отказан'. Желаете ли да продължите?",
                "Отказване на акт") == false)
            {
                return;
            }

            this.Cursor = Cursors.WaitCursor;
            try
            {
                if (Entity.Status.StatusCode == web.PropertyCertificateStatusCodes.SendedForApproval)
                {
                    Entity = service.ChangeStatus(Entity,
                        web.PropertyCertificateStatusCodes.ApprovalRejected);
                }
                else if (Entity.Status.StatusCode == web.PropertyCertificateStatusCodes.Approved)
                {
                    Entity = service.ChangeStatus(Entity,
                        web.PropertyCertificateStatusCodes.SignRejected);
                }

                BindToObject(Entity);
                Entity.RefreshBindedControls();
                SetControlStatus();
            }
            finally
            {
                this.Cursor = Cursors.Default;
            }
        }

        [PrincipalPermission(SecurityAction.Demand,
           Role = UserCredentials.PropertyCertificateDelete, Authenticated = true)]        
        private void btnDelete_Click(object sender, EventArgs e)
        {
            if (ConfirmationDialog.Execute(
                "Текущият акт ще бъде изтрит. Желаете ли да продължите?",
                "Изтриване на акт") == false)
            {
                return;
            }

            this.Cursor = Cursors.WaitCursor;
            try
            {
                service.Delete(Entity);
                DialogResult = DialogResult.OK;
            }
            finally
            {
                this.Cursor = Cursors.Default;
            }
        }

        //[PrincipalPermission(SecurityAction.Demand,
        //   Role = UserCredentials.PropertyCertificateSave, Authenticated = true)]        
        private void btnScan_Click(object sender, EventArgs e)
        {
            bool canEditScannedDocuments = AppUtils.HasPermission( UserCredentials.PropertyCertificateSave ) &&
                ( Entity.Status.StatusCode != web.PropertyCertificateStatusCodes.SendedForApproval ) &&
                ( Entity.Status.StatusCode != web.PropertyCertificateStatusCodes.Approved ) &&
                ( Entity.Status.StatusCode != web.PropertyCertificateStatusCodes.Signed ) &&
                ( Entity.Status.StatusCode != web.PropertyCertificateStatusCodes.Registered ) &&
                ( Entity.Status.StatusCode != web.PropertyCertificateStatusCodes.RegistrationRejected );

            ScannedDocumentsForm scannedDocumentsForm = new ScannedDocumentsForm( "PropertyCertificate", canEditScannedDocuments );
            scannedDocumentsForm.Execute( this.Entity.ID );
        }

        private void PropertyCertificateEditForm_Shown(object sender, EventArgs e)
        {
            tabControl1.SelectedTab = tabPage2;
            tabControl1.SelectedTab = tabPage1;
        }
    }
}