﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Bonea.PropertyRegister.MunicipalityProperty.WindowsClient.PropertyCertificateReference;
using Bonea.PropertyRegister.MunicipalityProperty.WindowsClient.Reports;
using Bonea.PropertyRegister.Common;
using System.Security.Permissions;

namespace Bonea.PropertyRegister.MunicipalityProperty.WindowsClient
{
    public partial class PropertyCertificateListForm : Form
    {
        PropertyCertificateServiceWse mService;

        public PropertyCertificateListForm()
        {
            InitializeComponent();

            cbSearchType.SelectedIndex = 0;
            dtpFromDate.Value = DateTime.Now;
            dtpToDate.Value = DateTime.Now;

            mService = ServiceFactory.GetPropertyCertificateService();

            cbStatus.DataSource = mService.GetStatusList();
            cbStatus.DisplayMember = "Name";
            cbStatus.ValueMember = "ID";

            //Set control state
            btnEdit.Enabled = AppUtils.HasPermission(
                UserCredentials.PropertyCertificateView);
            btnPrint.Enabled = AppUtils.HasPermission(
                UserCredentials.PropertyCertificatePrint);
        }

        public DialogResult Execute()
        {
            DialogResult dlgResult = ShowDialog();

            return dlgResult;
        }

        [PrincipalPermission(SecurityAction.Demand,
           Role = UserCredentials.PropertyCertificateView, Authenticated = true)]        
        private void BindGrid()
        {
            object results = null;

            switch (cbSearchType.SelectedIndex)
            {
                case 0: //Search by Number
                    if (String.IsNullOrEmpty(txtSearchText.Text) == false)
                    {
                        results = mService.SearchByNumber(
                            Int32.Parse(txtSearchText.Text));
                    }
                    break;
                case 1: //Search by Date created
                    results = mService.SearchByDateCreated(dtpFromDate.Value,
                        dtpToDate.Value); ;
                    break;
                case 2: //Search by Status
                    results = mService.SearchByStatus((int)cbStatus.SelectedValue); ;
                    break;
                case 3: //Search by Approved by user
                    if (String.IsNullOrEmpty(txtSearchText.Text) == false)
                    {
                        results = mService.SearchByApprovedUser(txtSearchText.Text);
                    }
                    break;
                case 4: //Search by Signed by user
                    if (String.IsNullOrEmpty(txtSearchText.Text) == false)
                    {
                        results = mService.SearchBySignedUser(txtSearchText.Text);
                    }
                    break;
                default:
                    throw new Exception("Unknown search type");
            }

            bsPropertyCertificate.DataSource = results;
        }

        private void btnSearch_Click(object sender, EventArgs e)
        {
            this.Cursor = Cursors.WaitCursor;
            try
            {
                BindGrid();
            }
            finally
            {
                this.Cursor = Cursors.Default;
            }
        }

        [PrincipalPermission(SecurityAction.Demand,
           Role = UserCredentials.PropertyCertificateView, Authenticated = true)]
        private void btnEdit_Click(object sender, EventArgs e)
        {
            if (bsPropertyCertificate.Count <= 0)
            {
                return;
            }

            PropertyCertificate selectedEntity =
                (PropertyCertificate)bsPropertyCertificate.Current;

            PropertyCertificate entity = mService.Load(selectedEntity.ID);

            PropertyCertificateEditForm form = new PropertyCertificateEditForm();

            DialogResult dialogResult = form.Execute(entity);
            if (dialogResult == DialogResult.OK)
            {
                   BindGrid();
            }
        }

        private void cbSearchType_SelectedIndexChanged(object sender, EventArgs e)
        {
            switch (cbSearchType.SelectedIndex)
            {
                case 0: //Search by Number
                    cbStatus.Visible = false;
                    txtSearchText.Visible = true;
                    dtpFromDate.Visible = false;
                    dtpToDate.Visible = false;
                    lblFromText.Visible = false;
                    lblToText.Visible = false;
                    break;
                case 1: //Search by Date created
                    cbStatus.Visible = false;
                    txtSearchText.Visible = false;
                    dtpFromDate.Visible = true;
                    dtpToDate.Visible = true;
                    lblFromText.Visible = true;
                    lblToText.Visible = true;
                    break;
                case 2: //Search by Status
                    cbStatus.Visible = true;
                    txtSearchText.Visible = false;
                    dtpFromDate.Visible = false;
                    dtpToDate.Visible = false;
                    lblFromText.Visible = false;
                    lblToText.Visible = false;
                    break;
                case 3: //Search by Approved by user
                    cbStatus.Visible = false;
                    txtSearchText.Visible = true;
                    dtpFromDate.Visible = false;
                    dtpToDate.Visible = false;
                    lblFromText.Visible = false;
                    lblToText.Visible = false;
                    break;
                case 4: //Search by Signed by user
                    cbStatus.Visible = false;
                    txtSearchText.Visible = true;
                    dtpFromDate.Visible = false;
                    dtpToDate.Visible = false;
                    lblFromText.Visible = false;
                    lblToText.Visible = false;
                    break;
                default:
                    throw new Exception("Unknown search type");
            }
        }

        [PrincipalPermission(SecurityAction.Demand,
           Role = UserCredentials.PropertyCertificatePrint, Authenticated = true)]        
        private void btnPrint_Click(object sender, EventArgs e)
        {
            if (bsPropertyCertificate.Count <= 0)
            {
                return;
            }

            PropertyCertificate selectedEntity =
                (PropertyCertificate)bsPropertyCertificate.Current;

            PropertyCertificate entity = mService.Load(selectedEntity.ID);

            PropertyCertificateReportForm reportForm = 
                new PropertyCertificateReportForm();

            reportForm.Execute(entity);
        }
    }
}