﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Data;
using System.Configuration;
using NHibernate;

namespace Bonea.PropertyRegister.DataAccess
{
    public abstract class SessionManager
    {
        public SessionManager()
        {
        }

        static NHibernate.Cfg.Configuration mConfiguration = null;
        static volatile ISessionFactory mSessionFactory = null;
        static object mFactorySyncObject = new object();

        private const string HBM_ASSEMBLY_NAME_SECTION = "HibernateMappingAssemblies";

        public abstract ISession Session
        {
            get;
        }

        public IDbConnection SessionConnection
        {
            get { return Session.Connection; }
        }

        public ISession NewSession()
        {
            ISessionFactory sessionFactory = GetSessionFactory();

            return sessionFactory.OpenSession();
        }

        public abstract void CloseSession();

        public abstract bool IsActiveSession
        {
            get;
        }

        private ISessionFactory GetSessionFactory()
        {
            if (mSessionFactory == null)
            {
                lock (mFactorySyncObject)
                {
                    if (mSessionFactory == null)
                    {
                        mConfiguration = new NHibernate.Cfg.Configuration();

                        List<string> mappingAssembliesList = (List<string>)ConfigurationManager.GetSection(
                            HBM_ASSEMBLY_NAME_SECTION);
                        foreach (string assemblyName in mappingAssembliesList)
                        {
                            mConfiguration.AddAssembly(assemblyName);
                        }

                        mSessionFactory = mConfiguration.BuildSessionFactory();
                    }
                }
            }

            return mSessionFactory;
        }

        public void BeginTransaction(IsolationLevel isolation)
        {
            Session.BeginTransaction(isolation);
        }

        public void CommitTransaction()
        {
            Session.Transaction.Commit();
        }

        public void RollbackTransaction()
        {
            Session.Transaction.Rollback();
        }

        public bool IsActiveTransaction
        {
            get
            {
                return ((Session.Transaction != null) && (Session.Transaction.IsActive));
            }
        }
    }
}
